#! python3  # noqa: E265

"""
    Widget to locate cadastre feature
"""

# standard
import os

# PyQGIS
from qgis.core import QgsProject
from qgis.gui import QgisInterface, QgsFeatureListComboBox
from qgis.PyQt import uic
from qgis.PyQt.QtWidgets import QWidget

# project
from locator_grand_lyon.toolbelt.log_handler import PlgLogger

# ############################################################################
# ########## Globals ###############
# ##################################


# ############################################################################
# ########## Classes ###############
# ##################################

COMMUNE_LAYER = "commune"
COMMUNE_ID = "id_com"
COMMUNE_DISPLAY = "texte"

PARCELLE_LAYER = "parcelle"
PARCELLE_ID = "id_par"
PARCELLE_DISPLAY = "section+parcelle"

PARCELLE_HIST_LAYER = "parcellehistorique"
PARCELLE_HIST_ID = "idparcelle"
PARCELLE_HIST_DISPLAY = "idparcelledgi"


class CadastreLocatorWidget(QWidget):
    """Widget to locate cadastre feature"""

    def __init__(self, parent: QWidget = None, iface: QgisInterface = None) -> None:
        super().__init__(parent)
        uic.loadUi(
            os.path.dirname(os.path.realpath(__file__)) + "/wdg_cadastre_locator.ui",
            self,
        )
        self.log = PlgLogger().log

        self.iface = iface
        self._init_all_cbx_feature_list()

        self.cbx_feat_commune.currentFeatureChanged.connect(self.commune_changed)
        self.btn_parcelle.clicked.connect(self.locate_parcelle)
        self.btn_parcelle_historique.clicked.connect(self.locate_parcelle_historique)

        QgsProject.instance().layersAdded.connect(self._project_layer_updated)
        QgsProject.instance().layersRemoved.connect(self._project_layer_updated)

    def _init_all_cbx_feature_list(self) -> None:
        """Init all QgsFeatureListComboBox with expected layer"""
        self._init_cbx_feature_list(
            self.cbx_feat_commune,
            layer_name=COMMUNE_LAYER,
            identifier_field=COMMUNE_ID,
            display_expression=COMMUNE_DISPLAY,
        )
        self._init_cbx_feature_list(
            self.cbx_feat_parcelle,
            layer_name=PARCELLE_LAYER,
            identifier_field=PARCELLE_ID,
            display_expression=PARCELLE_DISPLAY,
        )
        self._init_cbx_feature_list(
            self.cbx_feat_parcelle_historique,
            layer_name=PARCELLE_HIST_LAYER,
            identifier_field=PARCELLE_HIST_ID,
            display_expression=PARCELLE_HIST_DISPLAY,
        )

        layer = QgsProject.instance().mapLayersByName(PARCELLE_LAYER)
        self.btn_parcelle.setEnabled(len(layer) != 0)

        layer = QgsProject.instance().mapLayersByName(PARCELLE_HIST_LAYER)
        self.btn_parcelle_historique.setEnabled(len(layer) != 0)

    def _init_cbx_feature_list(
        self,
        cbx_feat_list: QgsFeatureListComboBox,
        layer_name: str,
        identifier_field: str,
        display_expression: str,
    ) -> None:
        """Initialization of a QgsFeatureListComboBox.
        If the source layer is not available, component is disabled

        :param cbx_feat_list: component to initialize
        :type cbx_feat_list: QgsFeatureListComboBox
        :param layer_name: source layer name
        :type layer_name: str
        :param identifier_field: identifier field
        :type identifier_field: str
        :param display_expression: display expression
        :type display_expression: str
        """
        layer = QgsProject.instance().mapLayersByName(layer_name)
        if len(layer):
            cbx_feat_list.setSourceLayer(layer[0])
            cbx_feat_list.setIdentifierFields([identifier_field])
            cbx_feat_list.setDisplayExpression(display_expression)
            cbx_feat_list.setToolTip("")
            cbx_feat_list.setEnabled(True)
        else:
            cbx_feat_list.setToolTip(
                self.tr("Couche '{}' non disponible").format(layer_name)
            )
            cbx_feat_list.clearEditText()
            cbx_feat_list.setEnabled(False)

    def commune_changed(self) -> None:
        """Update parcelle layer filter for a specific commune"""
        id_values = self.cbx_feat_commune.identifierValues()
        if len(id_values):
            self.cbx_feat_parcelle.setFilterExpression(f"{COMMUNE_ID}={id_values[0]}")

    def locate_parcelle(self) -> None:
        """Zoom to selected parcelle"""
        self._zoom_to_selected_feat(self.cbx_feat_parcelle)

    def locate_parcelle_historique(self) -> None:
        """Zoom to selected parcelle historique"""
        self._zoom_to_selected_feat(self.cbx_feat_parcelle_historique)

    def _zoom_to_selected_feat(self, cbx_feat_list: QgsFeatureListComboBox) -> None:
        """Zoom to selected feature of QgsFeatureListComboBox"""
        layer = cbx_feat_list.sourceLayer()
        if layer is not None:
            feat_request = cbx_feat_list.currentFeatureRequest()
            layer.selectByIds([i.id() for i in layer.getFeatures(feat_request)])
            canvas = self.iface.mapCanvas()
            canvas.zoomToSelected(layer)

    def _project_layer_updated(self) -> None:
        """Init QgsFeatureListComboBox for layer update"""
        self._init_all_cbx_feature_list()
